/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule permission-operation-controller
 */

(function() {
var PermissionOperationController;
var prototype;
var superConstructor;
var PermissionOperationView = dixim.server.PermissionOperationView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var ApiClientServer = dixim.common.ApiClientServer;
var BlockUi = dixim.common.BlockUi;

/**
 * @class PermissionOperationController
 * @constructor
 * @param {Object} configs
 */
PermissionOperationController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(PermissionOperationController, EventTarget);
dixim.server.PermissionOperationController = PermissionOperationController;
prototype = PermissionOperationController.prototype;
superConstructor = PermissionOperationController.superConstructor;

var PREV = 'prev';
var SELECT = 'select';
var ALLOW = 'allow';
var DENY = 'deny';
var DELETE = 'delete';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'permission-operation'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('fetchAllowList', this._successFetchAllowList, this);
    this._model.addEventListener('fetchAllowListError', this._errorFetchAllowList, this);
    this._model.addEventListener('fetchDenyList', this._successFetchDenyList, this);
    this._model.addEventListener('fetchDenyListError', this._errorFetchDenyList, this);
    this._model.addEventListener('setPermission', this._successSetPermission, this);
    this._model.addEventListener('setPermissionError', this._errorSetPermission, this);
    this._model.addEventListener('removeEntries', this._successRemoveEntries, this);
    this._model.addEventListener('removeEntriesError', this._errorRemoveEntries, this);

    this._view = new PermissionOperationView(configs);
    this._view.addEventListener(PREV, this._clickPrev, this);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(DENY, this._clickDeny, this);
    this._view.addEventListener(ALLOW, this._clickAllow, this);
    this._view.addEventListener(DELETE, this._clickDelete, this);

    this._isAllowDevice = configs.isAllowDevice || false;
    this._parentContext = configs.parentContext;
    this._blockUi = new BlockUi();
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._fetchList();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchList = function() {
    this._blockUi.block();
    if (this._isAllowDevice) {
        this._model.fetchAllowList();
    } else {
        this._model.fetchDenyList();
    }
};

prototype._viewList = function(list) {
    var listLen = list.length;

    if (listLen === 0) {
        this._view.noDevice();
        return;
    }

    for (var i = 0; i < listLen; i += 1) {
        this._view.addDevice(list[i]);
    }
};

prototype._successFetchAllowList = function(e) {
    var list = e.args.AllowList;

    this._viewList(list);
    this._blockUi.unblock();
};

prototype._errorFetchAllowList = function(e) {
    this._view.error(_('server_permission_could_not_fetch_allow'));
    this._blockUi.unblock();
};

prototype._successFetchDenyList = function(e) {
    var list = e.args.DenyList;

    this._viewList(list);
    this._blockUi.unblock();
};

prototype._errorFetchDenyList = function(e) {
    this._view.error(_('server_permission_could_not_fetch_deny'));
    this._blockUi.unblock();
};

prototype._successSetPermission = function() {
    this._blockUi.unblock();
    this._updateList();
};

prototype._errorSetPermission = function() {
    var errorDescription = (this._isAllowDevice) ? _('server_permission_could_not_set_deny') : _('server_permission_could_not_set_allow');

    this._blockUi.unblock();
    this._updateList();
    this._errorDialog(errorDescription);
};

prototype._successRemoveEntries = function() {
    this._blockUi.unblock();
    this._updateList();
};

prototype._errorRemoveEntries = function() {
    var errorDescription = (this._isAllowDevice) ? _('server_permission_could_not_delete_allow') : _('server_permission_could_not_delete_deny');

    this._blockUi.unblock();
    this._updateList();
    this._errorDialog(errorDescription);
};

prototype._errorDialog = function(text) {
    var dialog = null;
    var query = {};

    query.id = this._id;
    query.title = (this._isAllowDevice) ? _('server_permission_menu_allow') : _('server_permission_menu_deny');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._clickSelect = function() {
    this._view.switchSelect();
};

prototype._clickDeny = function(e) {
    var macList = e.args.macList;
    var macListLen = macList.length;
    var query = {};

    if (macListLen === 0) {
        this._errorDialog(_('server_permission_no_select_deny'));
        return;
    }
    this._blockUi.block();
    query.Permission = 'Deny';
    query.MacList = macList;
    this._model.setPermission(query);
};

prototype._clickAllow = function(e) {
    var macList = e.args.macList;
    var macListLen = macList.length;
    var query = {};

    if (macListLen === 0) {
        this._errorDialog(_('server_permission_no_select_allow'));
        return;
    }
    this._blockUi.block();
    query.Permission = 'Allow';
    query.MacList = macList;
    this._model.setPermission(query);
};

prototype._clickDelete = function(e) {
    var macList = e.args.macList;
    var macListLen = macList.length;
    var query = {};

    if (macListLen === 0) {
        this._errorDialog(_('server_permission_no_select_delete'));
        return;
    }
    this._blockUi.block();
    query.Entries = macList;
    this._model.removeEntries(query);
};

prototype._updateList = function() {
    this._view.cleanList();
    this._fetchList();
};

}());
