(function() {
/**
 * @module event
 * @submodule event-target
 */

/**
 * EventTarget class provides an ability to dispatch
 * user-defined events to its subscribers.
 *
 * @class EventTarget
 * @constructor
 */
var EventTarget = function() {
    /**
    * @property listeners
    * @protected
    * @type {Object}
    */
    this._listeners = this._listeners || {};
};

/**
 * Finds an index of specified entry.
 *
 * @method findEntryIndex
 * @private
 */
var findEntryIndex = function(entries, entry) {
    var i;
    var len;

    len = entries.length;
    for (i = 0; i < len; i += 1) {
        if (entries[i].listener !== entry.listener) continue;
        if ((!!entries[i].context !== !!entry.context) ||
            (entries[i].context !== entry.context)) continue;
        if ((!!entries[i].userData !== !!entry.userData) ||
            (entries[i].userData !== entry.userData)) continue;
        return i;
    }
    return len;
};

/**
 * Adds an event listener for the event type specified.
 *
 * @method addEventListener
 * @param {String} type
 * @param {Function} listener
 * @param {Object} [context]
 * @param {Object} [userData]
 */
EventTarget.prototype.addEventListener = function(type, listener, context, userData) {
    var entry = {listener: listener, context: arguments[2], userData: arguments[3]};
    var entries;

    if (typeof this._listeners[type] === 'undefined') {
        this._listeners[type] = [entry];
    } else {
        entries = this._listeners[type];
        if (findEntryIndex(entries, entry) !== entries.length) return;
        entries.push(entry);
    }
};

/**
 * Removes event listeners for the event type specified.
 *
 * @method removeEventListener
 * @param {String} type
 * @param {Function} [listener]
 * @param {Object} [context]
 * @param {Object} [userData]
 */
EventTarget.prototype.removeEventListener = function(type, listener, context, userData) {
    var entries = this._listeners[type];
    var entry = {listener: arguments[1], context: arguments[2], userData: arguments[3]};
    var i;

    if (!!!type) return;
    if (typeof entries === 'undefined') return;
    if (typeof entry.listener === 'undefined') {
        delete this._listeners[type];
        return;
    }

    if ((i = findEntryIndex(entries, entry)) !== entries.length) {
        entries.splice(i, 1);
    }
};

/**
 * Dispatches an event to subscribers.
 *
 * @method dispatchEvent
 * @param {Event} event an event object to be dispatched.
 */
EventTarget.prototype.dispatchEvent = function(event) {
    var entries = this._listeners[event.type];
    var wildcards = this._listeners['*'];
    var listeners = [];
    var i;
    var len;
    var context;

    if (typeof entries !== 'undefined') {
        listeners = listeners.concat(entries);
    }

    if (typeof wildcards !== 'undefined') {
        listeners = listeners.concat(wildcards);
    }

    len = listeners.length;
    for (i = 0; i < len; i += 1) {
        context = listeners[i].context || this;
        event.userData = listeners[i].userData;
        listeners[i].listener.call(context, event);
    }
};

/**
 * Removes all event subscribers.
 *
 * @method removeAllEventListener
 */
EventTarget.prototype.removeAllEventListener = function() {
    this._listeners = {};
};

dixim.event.EventTarget = EventTarget;

}());
